var ActionTypes = {};
ActionTypes.empty = 0;
ActionTypes.start = 1;
ActionTypes.cancel = 2;
ActionTypes.success = 3;
ActionTypes.failure = 4;
ActionTypes[ActionTypes.empty] = "empty";
ActionTypes[ActionTypes.start] = "start";
ActionTypes[ActionTypes.cancel] = "cancel";
ActionTypes[ActionTypes.success] = "success";
ActionTypes[ActionTypes.failure] = "failure";
/**
 * @license Redux-Typed-Actions
 * (c) 2017 Amin Paks <amin.pakseresht@hotmail.com>
 * License: MIT
 */
var keys = Object.keys;
var ClassActionFactory = /** @class */ (function () {
    function ClassActionFactory() {
        this.suffixes = {};
        this.registry = {};
        /* tslint:disable:strict-type-predicates */
        if (typeof process !== 'undefined' && process.env != undefined && process.env.NODE_ENV === 'production') {
            this.env = 'production';
        }
        else {
            this.env = 'development';
        }
        /* tslint:enable:strict-type-predicates */
        this.setSuffixes({
            empty: '',
            start: '',
            cancel: ' Cancel',
            success: ' Success',
            failure: ' Failure',
        });
    }
    /**
     * @param {?} value
     * @return {?}
     */
    ClassActionFactory.prototype.setEnvironment = function (value) {
        this.env = value;
    };
    /**
     * @param {?} inputs
     * @return {?}
     */
    ClassActionFactory.prototype.setSuffixes = function (inputs) {
        for (var _i = 0, _a = keys(inputs); _i < _a.length; _i++) {
            var key = _a[_i];
            this.suffixes[key] = inputs[key];
        }
    };
    /**
     * @param {?} input
     * @return {?}
     */
    ClassActionFactory.prototype.getSuffix = function (input) {
        return this.suffixes[ActionTypes[input]];
    };
    /**
     * @param {?} name
     * @param {?} type
     * @return {?}
     */
    ClassActionFactory.prototype.registerTypeName = function (name, type) {
        var /** @type {?} */ actionTypeName = this.getTypeName(name, type);
        if (this.registry[actionTypeName]) {
            throw new Error("Action with type name as \"" + name + "\" has already registered. You need define unique actions!"); // `;
        }
        else {
            this.registry[actionTypeName] = true;
        }
    };
    /**
     * @param {?} name
     * @param {?} input
     * @return {?}
     */
    ClassActionFactory.prototype.getTypeName = function (name, input) {
        var /** @type {?} */ actionTypeName = name + this.getSuffix(input);
        return actionTypeName;
    };
    return ClassActionFactory;
}());
var factory = new ClassActionFactory();
/**
 * @license Redux-Typed-Actions
 * (c) 2017 Amin Paks <amin.pakseresht@hotmail.com>
 * License: MIT
 */
var defineProperties$1 = Object.defineProperties;
/**
 * @param {?} input
 * @return {?}
 */
function getSafeClassName(input) {
    var /** @type {?} */ inputAsString = input.toString();
    var /** @type {?} */ match = /^symbol\((.*)\)$/i.exec(inputAsString);
    if (match !== null) {
        inputAsString = "SymbolAction_" + match[1]; // `;
    }
    return inputAsString.replace(/\W/g, '_');
}
/**
 * @param {?} input
 * @return {?}
 */
function generateClassAction(input) {
    var /** @type {?} */ actionClassName = getSafeClassName(input);
    var /** @type {?} */ actionClass = (new Function("return function " + actionClassName + "() { }"))(); // `;
    return actionClass;
}
/**
 * @param {?} classAction
 * @param {?} actionTypeName
 * @param {?} getMethod
 * @return {?}
 */
function defineClassActionProperties(classAction, actionTypeName, getMethod) {
    var /** @type {?} */ isMethod = function (action) {
        return action.type === actionTypeName;
    };
    var /** @type {?} */ castMethod = function (action) {
        return action;
    };
    defineProperties$1(classAction, {
        /**
         * Defines a static property to get the "type" key
         * of this action
         */
        type: {
            writable: false,
            enumerable: true,
            configurable: false,
            value: actionTypeName,
        },
        /**
         * Defines a static method to check if a plain action
         * has the same type key as this defined action.
         */
        is: {
            writable: false,
            enumerable: false,
            configurable: false,
            value: isMethod,
        },
        /**
         * Returns the plain action passed as argument with proper typings
         */
        cast: {
            writable: false,
            enumerable: false,
            configurable: false,
            value: castMethod,
        },
        /**
         * Returns a plain action with a proper typed payload.
         */
        get: {
            writable: false,
            enumerable: false,
            configurable: false,
            value: getMethod,
        },
        /**
         * Returns a plain action with a proper typed payload.
         */
        strictGet: {
            writable: false,
            enumerable: false,
            configurable: false,
            value: getMethod,
        },
    });
}
/**
 * @license Redux-Typed-Actions v0.1.4
 * (c) 2017 Amin Paks <amin.pakseresht@hotmail.com>
 * License: MIT
 */
var defineProperties = Object.defineProperties;
/**
 *
 * @template Payload
 * @param {?} actionTypeName
 * @return {?}
 */
function defineAction(actionTypeName) {
    var /** @type {?} */ prefixedActionTypeName = factory.getTypeName(actionTypeName, ActionTypes.empty);
    var /** @type {?} */ classAction = generateClassAction(prefixedActionTypeName);
    var /** @type {?} */ getMethod = function (payload, meta, error) {
        if (payload === void 0) { payload = undefined; }
        if (meta === void 0) { meta = undefined; }
        if (error === void 0) { error = false; }
        return {
            type: actionTypeName,
            payload: payload,
            meta: meta,
            error: error,
        };
    };
    defineClassActionProperties(classAction, actionTypeName, getMethod);
    return /** @type {?} */ (classAction);
}
/**
 *
 * @template Payload, SuccessPayload, FailurePayload, CancelPayload
 * @param {?} actionTypeName
 * @return {?}
 */
function defineScenarioAction(actionTypeName) {
    var /** @type {?} */ startActionTypeName = factory.getTypeName(actionTypeName, ActionTypes.start);
    var /** @type {?} */ cancelActionTypeName = factory.getTypeName(actionTypeName, ActionTypes.cancel);
    var /** @type {?} */ successActionTypeName = factory.getTypeName(actionTypeName, ActionTypes.success);
    var /** @type {?} */ failureActionTypeName = factory.getTypeName(actionTypeName, ActionTypes.failure);
    var /** @type {?} */ startClassAction = generateClassAction(startActionTypeName);
    var /** @type {?} */ cancelClassAction = generateClassAction(cancelActionTypeName);
    var /** @type {?} */ successClassAction = generateClassAction(successActionTypeName);
    var /** @type {?} */ failureClassAction = generateClassAction(failureActionTypeName);
    var /** @type {?} */ getMethod = function (type, error, payload, meta) {
        if (type === void 0) { type = ''; }
        if (error === void 0) { error = false; }
        if (payload === void 0) { payload = undefined; }
        if (meta === void 0) { meta = undefined; }
        return {
            type: type,
            payload: payload,
            meta: meta,
            error: error,
        };
    };
    defineClassActionProperties(startClassAction, startActionTypeName, getMethod.bind(null, startActionTypeName, false));
    defineClassActionProperties(cancelClassAction, cancelActionTypeName, getMethod.bind(null, cancelActionTypeName, false));
    defineClassActionProperties(successClassAction, successActionTypeName, getMethod.bind(null, successActionTypeName, false));
    defineClassActionProperties(failureClassAction, failureActionTypeName, getMethod.bind(null, failureActionTypeName, true));
    defineProperties(startClassAction, {
        /**
         * Returns a simple class action for cancel
         */
        cancel: {
            writable: false,
            enumerable: false,
            configurable: false,
            value: cancelClassAction,
        },
        /**
         * Returns a simple class action for success
         */
        success: {
            writable: false,
            enumerable: false,
            configurable: false,
            value: successClassAction,
        },
        /**
         * Returns a simple class action for failure
         */
        failure: {
            writable: false,
            enumerable: false,
            configurable: false,
            value: failureClassAction,
        },
    });
    return /** @type {?} */ (startClassAction);
}
/**
 *
 * @template P
 * @param {?} actionTypeName
 * @return {?}
 */
function defineSymbolAction(actionTypeName) {
    return /** @type {?} */ (defineAction(/** @type {?} */ (Symbol(actionTypeName))));
}
/**
 * Generated bundle index. Do not edit.
 */
export { factory, ClassActionFactory as __ClassActionFactory, ActionTypes, defineAction, defineScenarioAction, defineSymbolAction };
//# sourceMappingURL=redux-typed-actions.js.map