export declare type ActionTypeKey = string | symbol;
export interface PlainAction<P = any, TK extends ActionTypeKey = any> {
    type: TK;
    payload: P;
    meta: string;
    error: boolean;
}
export interface ClassAction<P, TK extends ActionTypeKey> {
    type: TK;
    is(action: PlainAction): action is PlainAction<P, TK>;
    cast<TCast = P>(action: PlainAction): PlainAction<TCast, TK>;
    get(payload?: P, meta?: string, error?: boolean): PlainAction<P, TK>;
    strictGet(payload: P, meta?: string, error?: boolean): PlainAction<P, TK>;
}
export interface SimpleClassAction<P, TK extends ActionTypeKey> extends ClassAction<P, TK> {
    get(payload?: P, meta?: string): PlainAction<P, TK>;
    strictGet(payload: P, meta?: string): PlainAction<P, TK>;
}
export interface ClassScenarioAction<P, SP, FP, CP, TK extends ActionTypeKey> extends ClassAction<P, TK> {
    cancel: SimpleClassAction<CP, TK>;
    success: SimpleClassAction<SP, TK>;
    failure: SimpleClassAction<FP, TK>;
    get(payload?: P, meta?: string): PlainAction<P, TK>;
    strictGet(payload: P, meta?: string): PlainAction<P, TK>;
}
/**
 *
 * @param {string} actionTypeName A string or a symbol to represent this action
 * @return {ClassAction<Payload, string>} Returns a class action that generates actions with string type key
 */
export declare function defineAction<Payload = undefined>(actionTypeName: string): ClassAction<Payload, string>;
/**
 *
 * @param {string} actionTypeName A string or a symbol to represent this action
 * @return {ClassScenarioAction<Payload, SuccessPayload, FailurePayload, string>} Returns a scenario class action that generates actions with string type key
 */
export declare function defineScenarioAction<Payload = undefined, SuccessPayload = string, FailurePayload = string, CancelPayload = never>(actionTypeName: string): ClassScenarioAction<Payload, SuccessPayload, FailurePayload, CancelPayload, string>;
/**
 *
 * @param {string} actionTypeName A string or a symbol to represent this action
 * @return {ClassAction<P, symbol>} Returns a class action that generates actions with symbol type key
 */
export declare function defineSymbolAction<P = undefined>(actionTypeName: string): SimpleClassAction<P, symbol>;
